## ----echo=TRUE, message=FALSE, warning=FALSE---------------------------------------------------------------------------------
# Importing packages
library(data.table)
library(magrittr)
library(lubridate)
library(ggplot2)
library(cowplot)
library(repr)
library(kableExtra)



## ----------------------------------------------------------------------------------------------------------------------------
# polices sold between start 2016 to end 2017
dt_policydates <- data.table(date_UW = seq(as.Date("2016/1/1"), as.Date("2017/12/31"), "day"))

# number of policies per day follows Poisson process with mean 700 (approx 255,500 pols per annum)
dt_policydates[, ':='(policycount = rpois(.N,700),
                      date_lapse = date_UW %m+% years(1),
                      expodays = as.integer(date_UW %m+% years(1) - date_UW),
                      pol_prefix = year(date_UW)*10000 + month(date_UW)*100 + mday(date_UW))]




## ----------------------------------------------------------------------------------------------------------------------------
# Add columns defining Policy Covers   
dt_policydates[, Cover_B := round(policycount * 0.25)]
dt_policydates[, Cover_BO := round(policycount * 0.45)]
dt_policydates[, Cover_BOT := policycount - Cover_B - Cover_BO]

kable(head(dt_policydates), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")



## ----------------------------------------------------------------------------------------------------------------------------
# repeat rows for each policy by UW-Date
dt_policy <- dt_policydates[rep(1:.N, policycount),c("date_UW", "pol_prefix"), with = FALSE][,pol_seq:=1:.N, by=pol_prefix]

# Create a unique policy number 
dt_policy[, pol_number := as.character(pol_prefix * 10000 + pol_seq)]

kable(head(dt_policy), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")


## ----------------------------------------------------------------------------------------------------------------------------
# set join keys
setkey(dt_policy,'date_UW')
setkey(dt_policydates,'date_UW')  

# remove pol_prefix before join
dt_policydates[, pol_prefix := NULL]  

# join cover from summary file (dt_policydates)
dt_policy <- dt_policy[dt_policydates]  

# now create Cover field for each policy row
dt_policy[,Cover := 'BO']
dt_policy[pol_seq <= policycount- Cover_BO,Cover := 'BOT']
dt_policy[pol_seq <= Cover_B,Cover := 'B']  

dt_policy[, Cover_B := as.factor(Cover)]  

# remove interim calculation fields
dt_policy[, ':='(pol_prefix = NULL,
                 policycount = NULL,
                 pol_seq = NULL,
                 Cover_B = NULL,
                 Cover_BOT = NULL,
                 Cover_BO = NULL)]

# check output
kable(head(dt_policy), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")



## ----------------------------------------------------------------------------------------------------------------------------
# Add remaining policy details
dt_policy[, Brand := rep(rep(c(1,2,3,4), c(9,6,3,2)), length.out = .N)]
dt_policy[, Base_Price := rep(rep(c(600,550,300,150), c(9,6,3,2)), length.out = .N)]

# models types and model cost multipliers
for (eachBrand in unique(dt_policy$Brand)) {
  dt_policy[Brand == eachBrand, Model := rep(rep(c(3,2,1,0), c(10, 7, 2, 1)), length.out = .N)]
  dt_policy[Brand == eachBrand, Model_mult := rep(rep(c(1.15^3, 1.15^2, 1.15^1, 1.15^0), c(10, 7, 2, 1)), length.out = .N)]
}

dt_policy[, Price := ceiling (Base_Price * Model_mult)]

# check output
kable(head(dt_policy), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")



## ----------------------------------------------------------------------------------------------------------------------------
# colums to keep
cols_policy <- c("pol_number",
                 "date_UW",
                 "date_lapse",
                 "Cover",
                 "Brand",
                 "Model",
                 "Price")

dt_policy <- dt_policy[, cols_policy, with = FALSE]

# check output
kable(head(dt_policy), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")

save(dt_policy, file = "./dt_policy.rda")


## ----------------------------------------------------------------------------------------------------------------------------
# All policies have breakage cover
# claims uniformly sampled from policies
claim <- sample(nrow(dt_policy), size = floor(nrow(dt_policy) * 0.15))

# Claim severity multiplier sampled from beta distribution
dt_claim <- data.table(pol_number = dt_policy[claim, pol_number],
                       claim_type = 'B',
                       claim_count = 1,
                       claim_sev = rbeta(length(claim), 2,5))

# check output
kable(head(dt_claim), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")



## ----------------------------------------------------------------------------------------------------------------------------
# identify all policies with Oxidation cover
cov <- which(dt_policy$Cover != 'B')

# sample claims from policies with cover
claim <- sample(cov, size = floor(length(cov) * 0.05))

# add claims to table 
dt_claim <- rbind(dt_claim,
                  data.table(pol_number = dt_policy[claim, pol_number],
                             claim_type = 'O',
                             claim_count = 1,
                             claim_sev = rbeta(length(claim), 5,3)))

# check output
kable(head(dt_claim), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")



## ----------------------------------------------------------------------------------------------------------------------------
# identify all policies with Theft cover
# for Theft claim frequency varies by Brand
# So need to consider each in turn...

for(myModel in 0:3) {

    cov <- which(dt_policy$Cover == 'BOT' & dt_policy$Model == myModel)
    claim <- sample(cov, size = floor(length(cov) * 0.05*(1 + myModel)))
  
    dt_claim <- rbind(dt_claim,
                      data.table(pol_number = dt_policy[claim, pol_number],
                                 claim_type = 'T',
                                 claim_count = 1,
                                 claim_sev = rbeta(length(claim), 5,.5)))
}

# check output
kable(head(dt_claim), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")
kable(tail(dt_claim), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")



## ----------------------------------------------------------------------------------------------------------------------------
# set join keys
setkey(dt_policy, pol_number)
setkey(dt_claim, pol_number)

#join Brand and Price from policy to claim
dt_claim[dt_policy,
         on = 'pol_number',
         ':='(date_UW = i.date_UW,
              Price = i.Price,
              Brand = i.Brand)]

# check output
kable(head(dt_claim), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")



## ----------------------------------------------------------------------------------------------------------------------------

# use lubridate %m+% date addition operator 
dt_claim[, date_lapse := date_UW %m+% years(1)]
dt_claim[, expodays := as.integer(date_lapse - date_UW)]
dt_claim[, occ_delay_days := floor(expodays * runif(.N, 0,1))]

dt_claim[ ,delay_report := floor(365 * rbeta(.N, .4, 10))]  
dt_claim[ ,delay_pay := floor(10 + 40* rbeta(.N, 7,7))]  

dt_claim[, date_occur := date_UW %m+% days(occ_delay_days)]
dt_claim[, date_report := date_occur %m+% days(delay_report)]
dt_claim[, date_pay := date_report %m+% days(delay_pay)]

dt_claim[, claim_cost := round(Price * claim_sev)]

# check output
kable(head(dt_claim), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")



## ----------------------------------------------------------------------------------------------------------------------------
# add a unique claimkey based upon occurence date

dt_claim[, clm_prefix := year(date_occur)*10000 + month(date_occur)*100 + mday(date_occur)]
dt_claim[, clm_seq := seq_len(.N), by = clm_prefix]
dt_claim[, clm_number := as.character(clm_prefix * 10000 + clm_seq)]


# keep only first claim against policy (competing hazards)
setkeyv(dt_claim, c("pol_number", "clm_prefix"))
dt_claim[, polclm_seq := seq_len(.N), by = .(pol_number)]
dt_claim <- dt_claim[polclm_seq == 1,]

# colums to keep
cols_claim <- c("clm_number",
                "pol_number",
                "claim_type",
                "claim_count",
                "claim_sev",
                "date_occur",
                "date_report",
                "date_pay",
                "claim_cost")

dt_claim <- dt_claim[, cols_claim, with = FALSE]

# check output
kable(head(dt_claim), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")

save(dt_claim, file = "./dt_claim.rda")



## ----echo=FALSE--------------------------------------------------------------------------------------------------------------

dt_policy[, Cover2 := as.integer(factor(Cover))]

p1 <- ggplot(data = dt_policy, aes(x = Brand, y = ..prop..)) + 
  geom_bar(stat = "count") + 
  labs(title = "Phone Brand stationary distribution", x = NULL) + 
  scale_x_continuous(breaks=c(1,2,3, 4), labels=c("Brand 1", "Brand 2", "Brand 3", "Brand 4"))

p2 <- ggplot(dt_claim, aes(x=claim_cost, fill=claim_type, color=claim_type)) +
  geom_histogram(position="identity", alpha=0.5, binwidth = 10) + 
  labs(title = "Claim cost frequencies by damage") + 
  theme(legend.position = c(.65,.95),
        legend.direction = "horizontal",
        legend.background = element_rect(fill=alpha('white', 0.6)))

p3 <- ggplot(data = dt_policy, aes(x = Cover2, y = ..prop..)) + 
  geom_bar(stat = "count") + 
  labs(title = "Phone cover stationary distribution", x = NULL) + 
  scale_x_continuous(breaks=c(1,2,3), labels=c("B", "BO", "BOT"))

p4 <- ggplot(data = dt_policy, aes(x = format(date_UW, "%y-%m"), fill=factor(Model, levels = c(3,2,1,0)))) + 
  geom_bar(stat = "count") + 
  labs(title = "Policy count by model", x = NULL, fill = "Model") +
  theme(axis.text.x=element_text(angle=60, hjust=1),
        legend.position = c(.55,.75),
        legend.direction = "horizontal",
        legend.background = element_rect(fill=alpha('white', 0.6)))

plot_grid(p1,p2,p3,p4, labels = "AUTO")


## ----------------------------------------------------------------------------------------------------------------------------

dt_claim[, Days_reportdelay := as.numeric(difftime(date_report, date_occur, units="days"))]


hist(dt_claim[, Days_reportdelay],breaks = 50)



## ----------------------------------------------------------------------------------------------------------------------------
dt_claim[, Days_paymentdelay := as.numeric(difftime(date_pay, date_report, units="days"))]
hist(dt_claim[, Days_paymentdelay],breaks = 60)


## ----echo=FALSE--------------------------------------------------------------------------------------------------------------
## Exhibits to check output

tmp <- dt_policy[,.(Price = mean(Price)), by = .(Brand = paste0("Brand ", Brand), date_UW=format(date_UW, "%y-%m"))]
p1 <- ggplot(data = tmp, aes(x = date_UW, y = Price, group=factor(Brand))) + 
          geom_line(aes(color = Brand)) + 
          labs(title = "Average Phone Price", x = "UW Month")+
          theme(axis.text.x=element_text(angle=60, hjust=1),
                legend.position = c(.45,.95),
                legend.direction = "horizontal",
                legend.title = element_blank(),
                legend.background = element_rect(fill=alpha('white', 0.6)),
                legend.text = element_text(size = 6)) +
          ylim(0, 1000)

#function to calc policies in force on a date
policy_count <- function(mydate, myBrand){
  dt_policy[date_UW <= mydate & date_lapse >= mydate & Brand == myBrand, .N]
}

#function return every nth axis label
every_nth = function(n) {
  return(function(x) {x[c(TRUE, rep(FALSE, n - 1))]})
}


#create a table to store policy exposures by brand
dt_exposuredates <- NULL
for (eachBrand in unique(dt_policy$Brand)){
  dt_exposuredates <- rbind(dt_exposuredates, data.table(date_Expo = seq(as.Date("2016/1/1"), as.Date("2018/12/31"), "day"),
                                                         Brand = eachBrand))
}

# calculate the exposure
dt_exposuredates[, Expo := mapply(policy_count, date_Expo,Brand)]
# calculate monthly summaries
dt_exposure <- dt_exposuredates[, .(Exposure = sum(Expo)), keyby = .(date_occur=format(date_Expo, "%y-%m"), Brand = paste0("Brand ", Brand)) ]

# add Brand details to Claims details and aggregate to monthly summary
tmp <- dt_policy[dt_claim][,.(claim_cost = sum(claim_cost), claim_count = .N), keyby = .(date_occur=format(date_occur, "%y-%m"), Brand = paste0("Brand ", Brand))]

# add monthly exposures and calculate claim rate
tmp2 <- dt_exposure[tmp][, Claim_Rate := 12 * 30.5 * claim_count / Exposure]

p2 <- ggplot(data = tmp2, aes(x = date_occur, y = Claim_Rate, group=factor(Brand))) + 
  geom_line(aes(color = Brand)) + 
  labs(title = "Average Claim Rate by Brand", x = "Occurrence Month") +
  theme(axis.text.x=element_text(angle=60, hjust=1),
        legend.position = c(.45,.9),
        legend.direction = "horizontal",
        legend.title = element_blank(),
        legend.background = element_rect(fill=alpha('white', 0.6)),
        legend.text = element_text(size = 6)) +
  scale_x_discrete(breaks = every_nth(n = 3)) +
  ylim(0.15, 0.3)


tmp <- dt_claim[,.(claim_cost = mean(claim_cost)), keyby = .(date_occur=format(date_occur, "%y-%m"))]
p3 <- ggplot(data = tmp, aes(date_occur, claim_cost,  group = 1 )) + 
  geom_line() + 
  labs(title = "Average Claim Cost", x = "Occurence Month") +
  theme(axis.text.x=element_text(angle=60, hjust=1))+
  scale_x_discrete(breaks = every_nth(n = 3)) +
  ylim(0, 500)


tmp <- dt_claim[,.(claim_cost = sum(claim_cost)), keyby = .(date_occur=format(date_occur, "%y-%m"))]
p4 <- ggplot(data = tmp, aes(date_occur, claim_cost,  group = 1 )) + 
  geom_line() + 
  labs(title = "Total Claim Cost", x = "Occurence Month") +
  theme(axis.text.x=element_text(angle=60, hjust=1)) +
  scale_x_discrete(breaks = every_nth(n = 3)) 


plot_grid(p1,p2,p3,p4, labels = "AUTO")



## ----include=FALSE-----------------------------------------------------------------------------------------------------------

simulate_central_scenario <- function(seed = 1234){
  
  #seed = 1234  
  set.seed(seed)
  
  # Policy data
  #~~~~~~~~~~~~~~~~~
  
  # polices sold between start 2016 to end 2017
  dt_policydates <- data.table(date_UW = seq(as.Date("2016/1/1"), as.Date("2017/12/31"), "day"))
  
  # number of policies per day follows Poisson process with mean 700 (approx 255,500 pols per annum)
  dt_policydates[, ':='(policycount = rpois(.N,700),
                        date_lapse = date_UW %m+% years(1),
                        expodays = as.integer(date_UW %m+% years(1) - date_UW),
                        pol_prefix = year(date_UW)*10000 + month(date_UW)*100 + mday(date_UW))]
  
  
  # Add columns defining Policy Covers   
  dt_policydates[, Cover_B := round(policycount * 0.25)]
  dt_policydates[, Cover_BO := round(policycount * 0.45)]
  dt_policydates[, Cover_BOT := policycount - Cover_B - Cover_BO]
  
  
  # repeat rows for each policy by UW-Date
  dt_policy <- dt_policydates[rep(1:.N, policycount),c("date_UW", "pol_prefix"), with = FALSE][,pol_seq:=1:.N, by=pol_prefix]
  
  # Create a unique policy number 
  dt_policy[, pol_number := as.character(pol_prefix * 10000 + pol_seq)]
  
  # set join keys
  setkey(dt_policy,'date_UW')
  setkey(dt_policydates,'date_UW')  
  
  # remove pol_prefix before join
  dt_policydates[, pol_prefix := NULL]  
  
  # join cover from summary file (dt_policydates)
  dt_policy <- dt_policy[dt_policydates]  
  
  # now create Cover field for each policy row
  dt_policy[,Cover := 'BO']
  dt_policy[pol_seq <= policycount- Cover_BO,Cover := 'BOT']
  dt_policy[pol_seq <= Cover_B,Cover := 'B']  
  
  # remove interim calculation fields
  dt_policy[, ':='(pol_prefix = NULL,
                   policycount = NULL,
                   pol_seq = NULL,
                   Cover_B = NULL,
                   Cover_BOT = NULL,
                   Cover_BO = NULL)]
  
  # Add remaining policy details
  dt_policy[, Brand := rep(rep(c(1,2,3,4), c(9,6,3,2)), length.out = .N)]
  dt_policy[, Base_Price := rep(rep(c(600,550,300,150), c(9,6,3,2)), length.out = .N)]
  
  # models types and model cost multipliers
  for (eachBrand in unique(dt_policy$Brand)) {
    dt_policy[Brand == eachBrand, Model := rep(rep(c(3,2,1,0), c(10, 7, 2, 1)), length.out = .N)]
    dt_policy[Brand == eachBrand, Model_mult := rep(rep(c(1.15^3, 1.15^2, 1.15^1, 1.15^0), c(10, 7, 2, 1)), length.out = .N)]
  }
  
  dt_policy[, Price := ceiling (Base_Price * Model_mult)]
  
  
  # colums to keep
  cols_policy <- c("pol_number",
                   "date_UW",
                   "date_lapse",
                   "Cover",
                   "Brand",
                   "Model",
                   "Price")
  
  dt_policy <- dt_policy[, cols_policy, with = FALSE]
  
  # check output
  head(dt_policy)
  
  #save(dt_policy, file = "./dt_policy.rda")
  
  
  # Claims data
  #~~~~~~~~~~~~~~~~~
  
 # All policies have breakage cover
  # claims uniformly sampled from policies
  claim <- sample(nrow(dt_policy), size = floor(nrow(dt_policy) * 0.15))
  
  # Claim serverity multiplier sampled from beta distn
  dt_claim <- data.table(pol_number = dt_policy[claim, pol_number],
                         claim_type = 'B',
                         claim_count = 1,
                         claim_sev = rbeta(length(claim), 2,5))
  
  # identify all policies with Oxidation cover
  cov <- which(dt_policy$Cover != 'B')
  
  # sample claims from policies with cover
  claim <- sample(cov, size = floor(length(cov) * 0.05))
  
  # add claims to table 
  dt_claim <- rbind(dt_claim,
                    data.table(pol_number = dt_policy[claim, pol_number],
                               claim_type = 'O',
                               claim_count = 1,
                               claim_sev = rbeta(length(claim), 5,3)))
  
  
  # identify all policies with Theft cover
  # for Theft claim frequency varies by Brand
  # So need to consider each in turn...
  
  for(myModel in 0:3) {
    
    cov <- which(dt_policy$Cover == 'BOT' & dt_policy$Model == myModel)
    claim <- sample(cov, size = floor(length(cov) * 0.05*(1 + myModel)))
    
    dt_claim <- rbind(dt_claim,
                      data.table(pol_number = dt_policy[claim, pol_number],
                                 claim_type = 'T',
                                 claim_count = 1,
                                 claim_sev = rbeta(length(claim), 5,.5)))
  }
  
  # set join keys
  setkey(dt_policy, pol_number)
  setkey(dt_claim, pol_number)
  
  #join Brand and Price from policy to claim
  dt_claim[dt_policy,
           on = 'pol_number',
           ':='(date_UW = i.date_UW,
                Price = i.Price,
                Brand = i.Brand)]
  
  # use lubridate %m+% date addition operator 
  dt_claim[, date_lapse := date_UW %m+% years(1)]
  dt_claim[, expodays := as.integer(date_lapse - date_UW)]
  dt_claim[, occ_delay_days := floor(expodays * runif(.N, 0,1))]
  
  dt_claim[ ,delay_report := floor(365 * rbeta(.N, .4, 10))]  
  dt_claim[ ,delay_pay := floor(10 + 40* rbeta(.N, 7,7))]  
  
  dt_claim[, date_occur := date_UW %m+% days(occ_delay_days)]
  dt_claim[, date_report := date_occur %m+% days(delay_report)]
  dt_claim[, date_pay := date_report %m+% days(delay_pay)]
  
  dt_claim[, claim_cost := round(Price * claim_sev)]
  
  dt_claim[, clm_prefix := year(date_report)*10000 + month(date_report)*100 + mday(date_report)]
  
  dt_claim[, clm_seq := seq_len(.N), by = clm_prefix]
  dt_claim[, clm_number := as.character(clm_prefix * 10000 + clm_seq)]
  
  # colums to keep
  cols_claim <- c("clm_number",
                  "pol_number",
                  "claim_type",
                  "claim_count",
                  "claim_sev",
                  "date_occur",
                  "date_report",
                  "date_pay",
                  "claim_cost")
  
  dt_claim <- dt_claim[, cols_claim, with = FALSE]
  
  output <- list()
  output$dt_policy <- dt_policy
  output$dt_claim <- dt_claim
  
  return(output)
  
  
}




## ----------------------------------------------------------------------------------------------------------------------------
tmp <- simulate_central_scenario(1234)

kable(head(tmp$dt_claim), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")


